<div class="alert alert-danger">
  <p>The UI is currently in read only mode.</p>

  <p>
    To change this, you'll need to set <code>Flipper::UI.configuration.read_only = false</code> wherever flipper is running from.
  </p>

  <p>
    Alternatively, you may be using the <code>Flipper::Adapters::ReadOnly</code> adapter. This will also set Flipper into read only mode.
  </p>
</div>
